@@ -60,6 +60,13 @@ static gboolean mute_login_sounds = FALSE;
 static gboolean gst_init_failed;
 #endif /* USE_GSTREAMER */
 
+#ifdef USE_HILDON
+#if !HILDON_CHECK_VERSION(1,0,0)
+/* Cannot play multiple sounds at once, because it causes hangage :o( */
+static gboolean sound_in_progress = FALSE ;
+#endif /* HILDON < 1.0.0 */
+#endif /* USE_HILDON */
+
 static const struct pidgin_sound_event sounds[PURPLE_NUM_SOUNDS] = {
 	{N_("Buddy logs in"), "login", "login.wav"},
 	{N_("Buddy logs out"), "logout", "logout.wav"},
@@ -72,7 +79,8 @@ static const struct pidgin_sound_event sounds[PURPLE_NUM_SOUNDS] = {
 	{N_("Others talk in chat"), "chat_msg_recv", "receive.wav"},
 	/* this isn't a terminator, it's the buddy pounce default sound event ;-) */
 	{NULL, "pounce_default", "alert.wav"},
-	{N_("Someone says your username in chat"), "nick_said", "alert.wav"}
+	{N_("Someone says your username in chat"), "nick_said", "alert.wav"},
+	{N_("Attention received"), "got_attention", "alert.wav"}
 };
 
 static gboolean
@@ -144,7 +152,7 @@ im_msg_received_cb(PurpleAccount *account, char *sender,
 				   char *message, PurpleConversation *conv,
 				   PurpleMessageFlags flags, PurpleSoundEventID event)
 {
-	if (flags & PURPLE_MESSAGE_DELAYED)
+	if (flags & PURPLE_MESSAGE_DELAYED || flags & PURPLE_MESSAGE_NOTIFY)
 		return;
 
 	if (conv==NULL)
@@ -199,7 +207,7 @@ chat_msg_received_cb(PurpleAccount *account, char *sender,
 {
 	PurpleConvChat *chat;
 
-	if (flags & PURPLE_MESSAGE_DELAYED)
+	if (flags & PURPLE_MESSAGE_DELAYED || flags & PURPLE_MESSAGE_NOTIFY)
 		return;
 
 	chat = purple_conversation_get_chat_data(conv);
@@ -212,11 +220,20 @@ chat_msg_received_cb(PurpleAccount *account, char *sender,
 		return;
 
 	if (flags & PURPLE_MESSAGE_NICK || purple_utf8_has_word(message, chat->nick))
+		/* This isn't quite right; if you have the PURPLE_SOUND_CHAT_NICK event disabled
+		 * and the PURPLE_SOUND_CHAT_SAY event enabled, you won't get a sound at all */
 		play_conv_event(conv, PURPLE_SOUND_CHAT_NICK);
 	else
 		play_conv_event(conv, event);
 }
 
+static void
+got_attention_cb(PurpleAccount *account, const char *who, 
+	PurpleConversation *conv, guint type, PurpleSoundEventID event)
+{
+	play_conv_event(conv, event);
+}
+
 /*
  * We mute sounds for the 10 seconds after you log in so that
  * you don't get flooded with sounds when the blist shows all
@@ -297,6 +314,10 @@ pidgin_sound_init(void)
 	purple_prefs_add_bool(PIDGIN_PREFS_ROOT "/sound/enabled/pounce_default", TRUE);
 	purple_prefs_add_path(PIDGIN_PREFS_ROOT "/sound/file/pounce_default", "");
 	purple_prefs_add_string(PIDGIN_PREFS_ROOT "/sound/theme", "");
+	purple_prefs_add_bool(PIDGIN_PREFS_ROOT "/sound/enabled/sent_attention", TRUE);
+	purple_prefs_add_path(PIDGIN_PREFS_ROOT "/sound/file/sent_attention", "");
+	purple_prefs_add_bool(PIDGIN_PREFS_ROOT "/sound/enabled/got_attention", TRUE);
+	purple_prefs_add_path(PIDGIN_PREFS_ROOT "/sound/file/got_attention", "");
 	purple_prefs_add_bool(PIDGIN_PREFS_ROOT "/sound/conv_focus", TRUE);
 	purple_prefs_add_bool(PIDGIN_PREFS_ROOT "/sound/mute", FALSE);
 	purple_prefs_add_path(PIDGIN_PREFS_ROOT "/sound/command", "");
@@ -343,6 +364,12 @@ pidgin_sound_init(void)
 	purple_signal_connect(conv_handle, "received-chat-msg",
 						gtk_sound_handle, PURPLE_CALLBACK(chat_msg_received_cb),
 						GINT_TO_POINTER(PURPLE_SOUND_CHAT_SAY));
+	purple_signal_connect(conv_handle, "got-attention", gtk_sound_handle,
+						PURPLE_CALLBACK(got_attention_cb),
+						  GINT_TO_POINTER(PURPLE_SOUND_GOT_ATTENTION));
+	/* for the time being, don't handle sent-attention here, since playing a
+	 sound would result induplicate sounds. And fixing that would require changing the
+	 conversation signal for msg-recv */	
 }
 
 static void
@@ -374,6 +401,11 @@ bus_call (GstBus     *bus,
 	case GST_MESSAGE_EOS:
 		gst_element_set_state(play, GST_STATE_NULL);
 		gst_object_unref(GST_OBJECT(play));
+#ifdef USE_HILDON
+#if !HILDON_CHECK_VERSION(1,0,0)
+		sound_in_progress = FALSE ;
+#endif /* HILDON < 1.0.0 */
+#endif /* USE_HILDON */
 		break;
 	case GST_MESSAGE_WARNING:
 		gst_message_parse_warning(msg, &err, NULL);
@@ -407,6 +439,12 @@ expire_old_child(gpointer data)
 }
 #endif
 
+#define PLAY_SOUND_NORMALLY
+#ifdef USE_HILDON
+#	if !HILDON_CHECK_VERSION(1,0,0)
+#		undef PLAY_SOUND_NORMALLY
+#	endif /* HILDON < 1.0.0 */
+#endif /* USE_HILDON */
 static void
 pidgin_sound_play_file(const char *filename)
 {
@@ -414,7 +452,11 @@ pidgin_sound_play_file(const char *filename)
 #ifdef USE_GSTREAMER
 	float volume;
 	char *uri;
+#ifdef PLAY_SOUND_NORMALLY
 	GstElement *sink = NULL;
+#else /* !PLAY_SOUND_NORMALLY */
+	GError *error = NULL ;
+#endif /* PLAY_SOUND_NORMALLY */
 	GstElement *play = NULL;
 	GstBus *bus = NULL;
 #endif
@@ -490,10 +532,34 @@ pidgin_sound_play_file(const char *filename)
 	if (gst_init_failed)  /* Perhaps do gdk_beep instead? */
 		return;
 #ifdef USE_HILDON
+#if !HILDON_CHECK_VERSION(1,0,0)
+	if (sound_in_progress)
+		return ;
+	volume = (((float)((CLAMP(purple_prefs_get_int(PIDGIN_PREFS_ROOT "/sound/volume"),0,100))) * 65535.0) / 100.0);
+
+	uri = g_strdup_printf ("gnomevfssrc location=file://%s ! wavparse ! audioconvert ! dsppcmsink volume=%d", filename, (guint)volume) ;
+	play = gst_parse_launch (uri, &error) ;
+	if (NULL == play) {
+		purple_debug_error ("sound", "Unable to create GStreamer Maemo pipeline%s%s.",
+			NULL == error ? "" : ": ",
+			NULL == error ? "" : error->message) ;
+		if (NULL != error)
+			g_error_free (error) ;
+		g_free (uri) ;
+		return ;
+	}
+	if (NULL != error) {
+		purple_debug_error ("sound", "GStreamer Maemo pipeline \"%s\" created, but with error: %s.\n", uri, error->message) ;
+		g_error_free (error) ;
+	}
+
+	sound_in_progress = TRUE ;
+#else /* HILDON > 1.0.0 */
 	volume = (float)(CLAMP(purple_prefs_get_int(PIDGIN_PREFS_ROOT "/sound/volume"),0,100)) / 25;
-#else /* !USE_HILDON */
-	volume = (float)(CLAMP(purple_prefs_get_int(PIDGIN_PREFS_ROOT "/sound/volume"),0,100)) / 50;
+#endif /* HILDON < 1.0.0 */
 #endif /* USE_HILDON */
+#ifdef PLAY_SOUND_NORMALLY
+	volume = (float)(CLAMP(purple_prefs_get_int(PIDGIN_PREFS_ROOT "/sound/volume"),0,100)) / 50;
 	if (!strcmp(method, "automatic")) {
 		sink = gst_element_factory_make("gconfaudiosink", "sink");
 	}
@@ -525,6 +591,7 @@ pidgin_sound_play_file(const char *filename)
 	g_object_set(G_OBJECT(play), "uri", uri,
 		                     "volume", volume,
 		                     "audio-sink", sink, NULL);
+#endif /* PLAY_SOUND_NORMALLY */
 
 	bus = gst_pipeline_get_bus(GST_PIPELINE(play));
 	gst_bus_add_watch(bus, bus_call, play);
@@ -541,18 +608,12 @@ pidgin_sound_play_file(const char *filename)
 #else /* _WIN32 */
 	purple_debug_info("sound", "Playing %s\n", filename);
 
-	if (G_WIN32_HAVE_WIDECHAR_API ()) {
+	{
 		wchar_t *wc_filename = g_utf8_to_utf16(filename,
 				-1, NULL, NULL, NULL);
 		if (!PlaySoundW(wc_filename, NULL, SND_ASYNC | SND_FILENAME))
 			purple_debug(PURPLE_DEBUG_ERROR, "sound", "Error playing sound.\n");
 		g_free(wc_filename);
-	} else {
-		char *l_filename = g_locale_from_utf8(filename,
-				-1, NULL, NULL, NULL);
-		if (!PlaySoundA(l_filename, NULL, SND_ASYNC | SND_FILENAME))
-			purple_debug(PURPLE_DEBUG_ERROR, "sound", "Error playing sound.\n");
-		g_free(l_filename);
 	}
 #endif /* _WIN32 */
 
@@ -595,6 +656,7 @@ pidgin_sound_play_event(PurpleSoundEventID event)
 				purple_debug_error("sound", "The file: (%s) %s\n from theme: %s, was not found or wasn't readable\n",
 							sounds[event].pref, filename, theme_name);
 				g_free(filename);
+				filename = NULL;
 			}
 		}
 
@@ -618,19 +680,16 @@ gboolean
 pidgin_sound_is_customized(void)
 {
 	gint i;
-	gchar *path, *file;
+	gchar *path;
+	const char *file;
 
 	for (i = 0; i < PURPLE_NUM_SOUNDS; i++) {
 		path = g_strdup_printf(PIDGIN_PREFS_ROOT "/sound/file/%s", sounds[i].pref);
-		file = g_strdup(purple_prefs_get_path(path));
+		file = purple_prefs_get_path(path);
 		g_free(path);
 
-		if (file && file[0] != '\0'){
-			g_free(file);
+		if (file && file[0] != '\0')
 			return TRUE;
-		}
-
-		g_free(file);
 	}
 
 	return FALSE;
